//
//  MLArrayController.swift
//  memoList
//
//  Created by narita on 2019/04/21.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

class MLArrayController: NSArrayController
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
        
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @IBInspectable var sortOrderKeyPath: String = "sortOrder"

    // 必ず "sortOrderKeyPath"の順序にする
    override var sortDescriptors: [NSSortDescriptor]
    {
        set(newValue)
        {
            super.sortDescriptors = newValue
        }
        
        get
        {
            let theSortDescriptor = NSSortDescriptor(key: self.sortOrderKeyPath, ascending: true)
            
            if super.sortDescriptors.contains(theSortDescriptor)
            {
                return self.sortDescriptors
            }
            else
            {
                return [theSortDescriptor] + super.sortDescriptors
            }
        }
    }
    
    override func insert(_ object: Any, atArrangedObjectIndex index: Int)
    {
        super.insert(object, atArrangedObjectIndex: index)
        self.updateSortOrder()
    }
    
    override func insert(contentsOf objects: [Any], atArrangedObjectIndexes indexes: IndexSet)
    {
        super.insert(contentsOf: objects, atArrangedObjectIndexes: indexes)
        self.updateSortOrder()
    }
    
    override func remove(atArrangedObjectIndex index: Int)
    {
        super.remove(atArrangedObjectIndex: index)
        self.updateSortOrder()
    }
    
    override func remove(atArrangedObjectIndexes indexes: IndexSet)
    {
        super.remove(atArrangedObjectIndexes: indexes)
        self.updateSortOrder()
    }

    // MARK: - Utility methods

    // 全てのsortOrderを、GUIで表示されている順番に更新する
    @objc open func updateSortOrder()
    {
        if let theArrangedObjects = self.arrangedObjects as? [NSObject]
        {
            theArrangedObjects.enumerated().forEach()
            {
                $0.1.setValue($0.0, forKeyPath: self.sortOrderKeyPath)
            }
        }
    }
    
    @objc open func indexForInsertion() -> Int
    {
        if selectionIndexes.count == 0
        {
            // 何も選択されていない場合は末尾を挿入点とする
            return (self.arrangedObjects as! [NSObject]).count
        }
        else
        {
            // 選択されている場合は、最後の選択箇所の後ろを挿入点とする
            return self.selectionIndexes.last! + 1
        }
    }
    
}
